////////////////////////////////////////////////////////////////////////////////
/*! \file       static_assert.hpp
    Header file for static_assert classes

    Version:    1.0                       <br>
    Copyright:  Orjan Westin 2010         <br>
    Licence:    BSD (see EOF comment)     <br>
    Contact:    orjan@coolcowstudio.co.uk <br>
    Download:   http://coolcowstudio.co.uk/source/cpp/utilities.zip
*///////////////////////////////////////////////////////////////////////////////

#ifndef COCOST_STATIC_ASSERT_H_
#define COCOST_STATIC_ASSERT_H_

//! Cool Cow Studio namespace
namespace CoCoSt 
{
//! Static assert class that will be compiled on a false expression
/*! Usage: static_assert< expression >::valid_expression(); 

    Will compile to a no-op function call if 'expression' is true, but generate
	  a compilation error "valid_expression not found" if 'expression' is false.
	
	 'expression' must be evaluatable at compile time.
*/
template <bool b> 
struct static_assert
{};

//! Static assert class that will be compiled on a true expression
/*! Usage: static_assert< expression >::valid_expression(); 

    Will compile to a no-op function call if 'expression' is true, but generate
	  a compilation error "valid_expression not found" if 'expression' is false.
	
	  'expression' must be evaluatable at compile time.
*/
template <> struct static_assert<true> 
{ 
  //! No-op function
	static void valid_expression() 
	{}; 
};
} // namespace

#endif // COCOST_STATIC_ASSERT_H_


/*

Copyright (c) 2010, Orjan Westin
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list
of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this
list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.

The name of the contributor(s) may be used to endorse or promote products derived
from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

